#################################s#o#q######################################
## by Arthur Chan @ soqhome.com
#############################################################################

from Products.SoQClub.config import *
from Products.SoQClub.Widget import *
from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo
from Acquisition import ImplicitAcquisitionWrapper, aq_parent, aq_base, aq_inner

#############################################################################
##                             Setting                                     ##
#############################################################################

schema = BaseFolderSchema +  Schema((
    StringField('title'
                ,searchable=1
                ,required=1
                ),
    TextField('description'
              ,searchable=1
              ,allowable_content_types=('text/html',)
              ,widget=TextAreaWidget(label="Teaser")
              ),
    StringField('contactemail'
                ,required=1
                ,validator='isEmail'
                ,widget=StringWidget(label="Contact Email")
                ),
    DateTimeField('datefrom'
              ,widget=CalendarWidget(label="Activity Starts")
              ),
    DateTimeField('dateto'
              ,widget=CalendarWidget(label="Activity Ends")
              ),
    ImageField('image'
              ,allowable_content_types=('image/*',)
              ,sizes=SOQIMAGE_SIZES
              ,widget=ImageWidget(
                           label="Title Image",
                           description="A title image is used to represent your activity")
              ),

    TextField('body'
              ,widget=VisualWidget(label="Content")
              ),
    CMFObjectField('multimedia'
              ,portal_type='SoQVideo'
              ,addable=1
              ,destination=''
              ,allowed_types=('SoQAudio', 'SoQVideo',)
              ,multiValued=1
              ,widget=SoQFieldWidget(
                      label="Multimedia Files",
                      is_description=1,
                      description="Put your multimedia onlines, it is a powerful tool to express your activity",
                      allowed_types_names=('SoQ Audio', 'SoQ Video',))
              ),
    CMFObjectField('photoalbum'
              ,portal_type='Photo Album'
              ,addable=1
              ,destination='${here}/photoalbum'
              ,allowed_types=('Photo', 'Photo Album',)
              ,multiValued=1
              ,widget=SoQFieldWidget(
                      allowed_actions=['view', 'edit', 'delete'],
                      label="Photos and Photo Albums",
                      allowed_types_names=('Photo', 'Photo Album',))
              ),
#    CMFObjectField('attachment',
#              portal_type='File',
#              addable=1,
#              destination='',
#              allowed_types=('File',),
#              multiValued=1,
#              widget=SoQFieldWidget(allowed_types_names=('File',)),
#              ),
    CMFObjectField(
              'form'
              ,portal_type='Form'
              ,addable=1
              ,destination=''
              ,allowed_types=('Form',)
              ,multiValued=1
              ,widget=SoQFieldWidget(
                      label='Form',
                      descripition="You can create your own application form online.",
                      allowed_types_names=('Form',))
              ),
    CMFObjectField('poll'
              ,portal_type='PlonePopoll'
              ,addable=1
              ,destination=''
              ,allowed_types=('PlonePopoll',)
              ,multiValued=1
              ,widget=SoQFieldWidget(
                             label="Poll",
                             description="Polling and voting",
                             allowed_types_names=('PlonePopoll',)),
              ),
    CMFObjectField('staff'
              ,portal_type='Wiki Folder'
              ,addable=0
              ,visible=0
              ,widget=SoQFieldWidget(
                      label="Staff Tools",
                      allowed_actions=[],
                      allowed_types_names=('Wiki Folder',))
              ),
    CMFObjectField('forum'
              ,portal_type='Forum'
              ,addable=0
              ,visible=0
              ,widget=SoQFieldWidget(
                      label="Forum",
                      allowed_actions=['view'],
                      allowed_types_names=('Forum',))
              ),
    LinesField("auxiliary"
              ,default=SOQAUXILIARY_NAME
              ,vocabulary=SOQAUXILIARY_NAME
              ,widget=MultiSelectionWidget(label="Select Auxiliary Slots",
                      is_description=1,
                      description="Select the auxiliaries you want to display in your activity. Select all if you are not sure",
                      format='checkbox',
                  )
              ),
    ))

class SoQActivity(BaseFolder):
    security = ClassSecurityInfo()
    schema = schema
    content_icon = 'icon_activity.gif'
    archetype_name = "SoQ Activity"
    allowed_content_types = SOQACTIVITY_ALLOW_CONTENT_TYPE
    global_allow = 0
    
    actions = (
        {
        'id': 'view',
        'name': 'View',
        'action': 'soqactivity_view',
        'permissions': (CMFCorePermissions.View,)
        },
        { 'id'            : 'forum'
        , 'name'          : 'Forum'
        , 'action'        : 'forum'
        , 'permissions'   :  (CMFCorePermissions.View,)
        , 'category'      : 'object'
        , 'visible':1
        },
        { 'id'            : 'order'
        , 'name'          : 'Order'
        , 'action'        : 'portal_form/PloneFolder_orderForm'
        , 'permissions'   :  (CMFCorePermissions.ModifyPortalContent,)
        , 'category'      : 'object'
        , 'visible':1
        },
        { 'id'            : 'metadata'
        , 'name'          : 'Properties'
        , 'action'        : 'portal_form/base_metadata'
        , 'permissions'   :  (CMFCorePermissions.ManageProperties,)
        , 'category'      : 'object'
        , 'visible': 0
        },
        { 'id'            : 'staff'
        , 'name'          : 'Staff'
        , 'action'        : 'staff'
        , 'permissions'   :  (CMFCorePermissions.ModifyPortalContent ,)
        , 'category'      : 'object'
        , 'visible':1
        },
        { 'id'            : 'local_roles'
        , 'name'          : 'Local Roles'
        , 'action'        : 'folder_localrole_form'
        , 'permissions'   : (CMFCorePermissions.ManageProperties,)
        },
        { 'id'            : 'references'
        , 'name'          : 'References'
        , 'action'        : 'portal_form/reference_edit'
        , 'permissions'   :  (CMFCorePermissions.ModifyPortalContent ,)
        , 'category'      : 'object'
        , 'visible':0
        },
        )

    #############################################################################
    ##                             Basics                                      ##
    #############################################################################

    def __init__(self
                , id
                , title=''
                , **kwarg):
        BaseFolder.__init__(self, id, title=title, **kwarg)
        
        
        ## Add products
        #CMFInit.addWikiFolder(self, 'wiki', 'staff discussion')
        self.manage_addProperty('left_slots', [], 'lines')
        self.manage_addProperty('right_slots', [], 'lines')
        

#    security.declarePublic( 'folder_contents' )
#    def folder_contents( self, REQUEST=None, RESPONSE=None, **kw ):
#        """
#            Return article contents view
#        """
#        
#        if REQUEST is None:
#            REQUEST = self.REQUEST
#              
#        view = self.soqactivity_view
#        if getattr(aq_base(view), 'isDocTemp', 0):
#            return apply(view, ( self, REQUEST  ) )
#        else:
#            return view()        
#
#    security.declarePublic( 'folder_listing' )
#    def folder_listing( self, REQUEST=None, RESPONSE=None, **kw ):
#        """
#            Return article view
#        """
#        
#        if REQUEST is None:
#            REQUEST = self.REQUEST
#              
#        view = self.soqactivity_view
#        if getattr(aq_base(view), 'isDocTemp', 0):
#            return apply(view, ( self, REQUEST  ) )
#        else:
#            return view()   

    security.declareProtected( CMFCorePermissions.ModifyPortalContent, 'setAuxiliary' )
    def setAuxiliary(self, value):
        schema = self.Schema()
        field = schema.get('auxiliary', None)
        
        field.set(instance=self, value=value)
        aux = value
        soqclub_tool = self.soqclub_tool
        ls = soqclub_tool.SOQAUXILIARY_SLOTS_BASICLEFT
        rs = soqclub_tool.SOQAUXILIARY_SLOTS_BASICRIGHT
        
        for key in SOQAUXILIARY_KEY:
            if key in aux:
                value = SOQAUXILIARY_SLOTS[key]
                pos = SOQAUXILIARY_SLOTSPOSITION[key]
                if pos =='l':
                    if value not in ls:
                        ls.append(value) 
                elif value not in rs:
                    rs.append(value) 
        self.manage_changeProperties({'left_slots': ls,
                                      'right_slots': rs,
                                      })

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        BaseFolder.manage_afterAdd(self, item, container)
        
        ## add properties
        value = getattr(self, 'auxiliary', [])
        self.setAuxiliary(value=value)
        
        
        
        
    #############################################################################
    ##                             Utils                                       ##
    #############################################################################

    security.declarePrivate( 'comparaison' )    
    def comparaison(self, x,y):
        if hasattr(self, 'order'):
            list_ref = self.order  
            if  x.getId() not in list_ref:
               return 1
            if  y.getId() not in list_ref:
               return -1
            xid=list_ref.index(x.getId())
            yid=list_ref.index(y.getId())
        else:
            xid =x.getId()
            yid = y.getId()
            
        if xid<yid:
            return -1
        elif xid>yid:
            return +1
        else:
            return 0

    #def sortByDate(self, a,b):
    #    if a.bobobase_modification_time() < b.bobobase_modification_time(): return -1
    #    if a.bobobase_modification_time() > b.bobobase_modification_time(): return 1
    #    return 0


    security.declareProtected(CMFCorePermissions.View, 'listActivityFolderContents')
    def listActivityFolderContents( self, spec=None, contentFilter=None, **kwargs ):
        #results = BaseFolder.listFolderContents( self, spec, contentFilter )
        results = BaseFolder.contentValues( self, spec, contentFilter )
        
        if kwargs.has_key('state'):
            state = kwargs['state']
        else:
            state = ''

        if kwargs.has_key('sort_on'):
            sort_on = kwargs['sort_on']
        else:
            sort_on = ''
        
        if kwargs.has_key('sort_order'):
            sort_order = kwargs['sort_order']
        else:
            sort_order = ''
            
        if state <> '':
            contents = []
            wf = self.portal_workflow
            for result in results:
                if wf.getInfoFor(result,'review_state', '') == state :
                    contents.append(result)
        else:
            contents = results
        
        
	
        contents=list(contents)
        if sort_on =='date':
            contents.sort(lambda x, y: cmp(x.title_or_id().lower(),y.title_or_id().lower()))
        elif sort_on <> '':
            # this one should be defined better, but 1st, I know not all types of sorting method, 2nd, it works for me
            #try: 
            #    contents.sort(lambda x, y: cmp(x[sort_on]().lower(),y[sort_on]().lower()))
            #except:
            contents.sort(self.comparaison)
        else:
            contents.sort(self.comparaison)
        if sort_order == 'reverse':
            contents.reverse()
        
        values=contents[:]
	del(contents)
	    
        return values
        
    security.declarePublic('getActivity')
    def getActivity(self):
        #purl = self.portal_url(relative=1)
        #parent =self
        #while parent.meta_type <> 'SoQActivity' and parent.absolute_url('relative') <> purl:
        #    parent = parent.aq_parent
        #return  parent
        return self

    security.declarePublic('getActivityPath')
    def getActivityPath(self, relative=1):
        #purl = self.portal_url(relative=1)
        #parent =self
        #while parent.meta_type <> 'SoQActivity' and parent.absolute_url('relative') <> purl:
        #    parent = parent.aq_parent
        if relative==1:
            ret = self.absolute_url('relative')
        else:
            ret = self.absolute_url()
        return  ret

    security.declarePublic('getType')
    def getType(self):
        return  'activiy'


    #############################################################################
    ##                             CMF Calendar SUPPORT                        ##
    #############################################################################
    
    def start(self):
       """ Return our start time as a string. """
       if getattr(self,'datefrom',None):
           ret = self.datefrom
       else:
           ret = self.created()
       return ret

    def end(self):
        """ Return our end time as a string. """
        if getattr(self,'dateto',None):
            if self.dateto < getattr(self,'datefrom',self.created()):
                ret = getattr(self,'datefrom',self.created())
            else:
                ret = self.dateto
        else:
            ret = self.created()
        return ret 
    
            
    #############################################################################
    ##                             CMF CATALOG SUPPORT                         ##
    #############################################################################
    
    security.declareProtected(CMFCorePermissions.View, 'SearchableText')
    def SearchableText(self):
        "Returns a concatination of all searchable text"
        # Should be overriden by portal objects
        return "%s %s " % (self.getTitle(), self.getDescription())
         
registerType(SoQActivity)
