#################################s#o#q######################################
## by Arthur Chan @ soqhome.com
############################################################################


from Products.SoQClub.config import *
from Products.SoQClub.Widget import *

from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions

from AccessControl import ClassSecurityInfo

from Acquisition import ImplicitAcquisitionWrapper, aq_parent, aq_base, aq_inner

from DateTime import DateTime 

import string

#############################################################################
##                             Setting                                     ##
#############################################################################

schema = BaseFolderSchema +  Schema((
    StringField('title',
                searchable=1,
                required=1,
                ),
    TextField('description',
              searchable=1,
              allowable_content_types=('text/html',),
              widget=TextAreaWidget,
              ),
    StringField('contactemail',
                required=1,
                validator='isEmail',
                widget=StringWidget(label="Contact Email"),
                ),
    ImageField('image',
              allowable_content_types=('image/*',),
              sizes=SOQIMAGE_SIZES,
              widget=ImageWidget(
                           label="Title Image",
                           description="A title image is used to represent your channel"),
              ),
    CMFObjectField('activity',
              portal_type='SoQActivity',
              addable=1,
              destination='',
              allowed_types=('SoQActivity',),
              multiValued=1,
              widget=SoQFieldWidget(label="Activity",
              	                     allowed_types_names=('SoQ Activity',)),
              ),
    CMFObjectField('poster',
              portal_type='SoQPoster',
              addable=1,
              destination='',
              allowed_types=('SoQPoster',),
              multiValued=1,
              widget=SoQFieldWidget(label="Poster",
                                 allowed_types_names=('SoQ Poster',)),
              ),
    CMFObjectField('event_news',
              portal_type='News Item',
              addable=1,
              destination='',
              allowed_types=('Event', 'News Item'),
              multiValued=1,
              widget=SoQFieldWidget(label="Event And News",
                             allowed_types_names=('Event', 'News Item',)),
              ),
    CMFObjectField('article',
              portal_type='Article',
              addable=1,
              destination='',
              allowed_types=('Article',),
              multiValued=1,
              widget=SoQFieldWidget(label="Article",
                             allowed_types_names=('Article',)),
              ),
    CMFObjectField('remotechannel',
              portal_type='SoQRemoteChannel',
              addable=1,
              destination='',
              allowed_types=('SoQRemoteChannel',),
              multiValued=1,
              widget=SoQFieldWidget(label="Subscribed Remote Channels",
                             allowed_actions=['edit', 'delete'],
                             description="""You can subscribe other station's channel content.
                                            Thus you will know their activities, events, and news.""",
                             allowed_types_names=('SoQ Remote Channel',)),
              ),
    ))

class SoQChannel(BaseFolder):
    security = ClassSecurityInfo()
    schema = schema
    content_icon = 'icon_channel.gif'
    archetype_name = "SoQ Channel"
    allowed_content_types = SOQCHANNEL_ALLOW_CONTENT_TYPE

    actions = ({
        'id': 'view',
        'name': 'View',
        'action': 'soqchannel_view',
        'permissions': (CMFCorePermissions.View,)
        },
        { 'id'            : 'order'
        , 'name'          : 'Order'
        , 'action'        : 'portal_form/PloneFolder_orderForm'
        , 'permissions'   :  (CMFCorePermissions.ModifyPortalContent,)
        , 'category'	  : 'object'
        , 'visible':1
        },
        { 'id'            : 'metadata'		# Nasty Hack! yup, i know, but gimme time or suggestion, thx man
        , 'name'          : 'Properties'
        , 'action'        : 'portal_form/base_metadata'
        , 'permissions'   :  (CMFCorePermissions.ManageProperties,)
        , 'category'	  : 'object'
        , 'visible':0
        },
        { 'id'            : 'references'	# Nasty Hack again! okay, your help will be great
        , 'name'          : 'References'
        , 'action'        : 'portal_form/reference_edit'
        , 'permissions'   :  (CMFCorePermissions.ModifyPortalContent,)
        , 'category'	  : 'object'
        , 'visible':0
        },
        { 'id'            : 'local_roles'
        , 'name'          : 'Local Roles'
        , 'action'        : 'folder_localrole_form'
        , 'permissions'   : (CMFCorePermissions.ManageProperties,)
        },
        { 'id'            : 'cluster'
        , 'name'          : 'cluster'
        , 'action'        : 'soqallchannels_select'
        , 'permissions'   :  (CMFCorePermissions.ModifyPortalContent,)
        , 'category'	  : 'object'
        , 'visible':1
        },
#        { 'id'            : 'syndication'
#        , 'name'          : 'Syndication'
#        , 'action'        : 'portal_form/synPropertiesForm'
#        , 'permissions'   :  (CMFCorePermissions.ManageProperties,)
#        , 'category'	  : 'object'
#        , 'visible':0
#        },
        )

    #############################################################################
    ##                             Basics                                      ##
    #############################################################################

    def __init__(self
                , id
                , title=''
                , **kwarg):
        BaseFolder.__init__(self, id, title=title, **kwarg)
        
        ## add properties
        soqclub_tool = self.soqclub_tool
        self.manage_addProperty('left_slots', soqclub_tool.SOQCHANNEL_SLOTS_BASICLEFT, 'lines')
        self.manage_addProperty('right_slots', soqclub_tool.SOQCHANNEL_SLOTS_BASICRIGHT, 'lines')        
  

            
    #############################################################################
    ##                             Utils                                       ##
    #############################################################################


    security.declarePublic( 'getClusterURL')
    def getClusterURL(self):
        soqclub_tool = self.soqclub_tool
        return soqclub_tool.SOQCHANNEL_CLUSTERMASTER

    security.declarePublic( 'getPublishType')
    def getPublishType(self):
        return SOQCHANNEL_PUBLISH_PORTAL_TYPE


    security.declarePrivate( 'comparaison')
    def comparaison(self, x,y):
        if hasattr(self, 'order'):
            list_ref = self.order  
            if  x.getId() not in list_ref:
               return 1
            if  y.getId() not in list_ref:
               return -1
            xid=list_ref.index(x.getId())
            yid=list_ref.index(y.getId())
        else:
            xid =x.getId()
            yid = y.getId()
            
        if xid<yid:
            return -1
        elif xid>yid:
            return +1
        else:
            return 0

    security.declareProtected(CMFCorePermissions.View, 'listChannelFolderContents')
    def listChannelFolderContents( self, spec=None, contentFilter=None, **kwargs ):
        #results = BaseFolder.listFolderContents( self, spec, contentFilter )
        results = BaseFolder.contentValues( self, spec, contentFilter )
        
        if kwargs.has_key('state'):
            state = kwargs['state']
        else:
            state = ''
            
        if kwargs.has_key('sort_on'):
            sort_on = kwargs['sort_on']
        else:
            sort_on = ''
            
        if kwargs.has_key('sort_order'):
            sort_order = kwargs['sort_order']
        else:
            sort_order = ''
        
        if state <> '':
            contents = []
            wf = self.portal_workflow
            for result in results:
                if wf.getInfoFor(result,'review_state', '') == state :
                    contents.append(result)
        else:
            contents = results
        
        
	
        contents=list(contents)
        
        if sort_on =='date':
            contents.sort(lambda x, y: cmp(x.title_or_id().lower(),y.title_or_id().lower()))
        elif sort_on <> '':
            # this one should be defined better, but 1st, I know not all types of sorting method, 2nd, it works for me
            #try: 
            #    contents.sort(lambda x, y: cmp(x[sort_on]().lower(),y[sort_on]().lower()))
            #except:
            contents.sort(self.comparaison)
        else:
            contents.sort(self.comparaison)
        if sort_order == 'reverse':
            contents.reverse()
        
        values=contents[:]
	del(contents)
	    
        return values

    def getChannel(self):
        return  self
            

    security.declarePublic('getChannelPath')
    def getChannelPath(self, relative=1):
        if relative==1:
            ret = self.absolute_url('relative')
        else:
            ret = self.absolute_url()
        return  ret


    def __init__(self
                , id
                , title=''
                , **kwarg):
        
        BaseFolder.__init__(self, id, title=title, **kwarg)
        
        ## add properties
        
        self.manage_addProperty('left_slots', [], 'lines')
        self.manage_addProperty('right_slots', [], 'lines')
        

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        BaseFolder.manage_afterAdd(self, item, container)
        
        soqclub_tool = self.soqclub_tool

        self.manage_changeProperties({'left_slots': soqclub_tool.SOQCHANNEL_SLOTS_BASICLEFT,
                                      'right_slots': soqclub_tool.SOQCHANNEL_SLOTS_BASICRIGHT,
                                      })
        if not hasattr(self, 'ClusterMaster'):
            url = soqclub_tool.SOQCHANNEL_CLUSTERMASTER
            req = {'id':'ClusterMaster', 'url':url,}
            self.manage_addProduct['XMLRPCProxy'].XMLRPCProxy.createInObjectManager('ClusterMaster',REQUEST=req)
	    rproxy = self['ClusterMaster']
            rproxy.url = url
            rproxy.ValidMethodList = SOQCHANNEL_CLUSTERMASTER_VALIDMETHODLIST
        
        if self.soqclub_tool.SOQCHANNEL_ENABLE_SYNDICATION:
            try: self.portal_syndication.enableSyndication(self)
            # should be a better catch, but for the moment
            # this will do
            except: print "Error enabling syndication"


    security.declareProtected( CMFCorePermissions.ModifyPortalContent, 'addRemoteChannel' )
    def addRemoteChannel(self, REQUEST=None):
        name= 'SoQRemoteChannel.' + DateTime().strftime('%Y-%m-%d.%I%M%S')
        if self.REQUEST <> None:
            r = self.REQUEST
            # reference http://www.python.org/doc/current/lib/module-time.html
            
            self.manage_addProduct['SoQClub'].addSoQRemoteChannel(id=name, REQUEST=r)
            #c = self[name]
            #if r.get('title', '')<>'':
            #    c.title = r.get('title', '')
            #if r.get('channelurl', '') <>'':
            #    c.channelurl = r.get('channelurl', '')
            #
            #return r.RESPONSE.redirect(c.absolute_url() + \
            #                                 "?portal_status_message=remote+channel+created")
            #
        



        
    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        BaseFolder.manage_beforeDelete(self, item, container)
        

        
    security.declarePublic('getType')
    def getType(self):
        return  'channel'


    #############################################################################
    ##                             Syndication                                 ##
    #############################################################################

    security.declareProtected( CMFCorePermissions.View, 'queryCatalog' )
    def queryCatalog( self, REQUEST=None, **kw ):
        query = {'path': self.absolute_url('relative'),
                'meta_type' :SOQCHANNEL_ALLOW_CONTENT_TYPE,
                'review_state':'published',}

        kw.update(query )
        return apply( self.portal_catalog.searchResults, ( REQUEST, ), kw )


    security.declarePublic('synContentValues')
    def synContentValues(self):
        """ play nice with portal_syndication_tool"""
        return self.queryCatalog()

registerType(SoQChannel)
